Unit Vesa;

Interface

Procedure GetVesaInfo;
Procedure GetModeInfo(ModeNr:Word);
Procedure InitVesaMode(ModeNr:Word);
Procedure InitTextMode;
Procedure SwitchTo(BankNr:Word);
Procedure SetLogScan(NewLength:Word);
Procedure SetDisplayXY(X,Y:Word);
Procedure PutPixel8(X,Y:Word; Col:Byte);
Procedure PutPixel32(X,Y:Word; Col:LongInt);
Function GetPixel8(X,Y:Word):Byte;

Function SetUpMode8(ModeNr:Word):Boolean;
Function SetUpMode32(ModeNr:Word):Boolean;
Function SetUpMode(X,Y:Word; BPB:Byte):Boolean;

Type TAvailModes=Array [0..255] Of Word;
     TModeAttr=(Avail,Reserved,BIOSSupport,Col,Graph,Bit5,Bit6,Bit7,LFBAvail);
     TWinAttr=(Support,Readable,Writeable);

Var VesaInfo:Record
                   Signature:Array [0..3] Of Char;             {VESA}
                   Version:Word;                               {Hi.Lo}
                   OEMString:PChar;
                   Capabilities:Array [0..3] Of Byte;
                   AvailModes:^TAvailModes;                    {terminated by $ffff}
                   VideoRam:Word;                              {*64 kB}
                   Reserved:Array [0..235] Of Byte;
             End;
    ModeInfo:Record
                   ModeAttr:Set Of TModeAttr;
                   WinAAttr:Set Of TWinAttr;
                   WinBAttr:Set Of TWinAttr;
                   WinGranularity:Word;
                   WinSize:Word;
                   WinASegment:Word;
                   WinBSegment:Word;
                   BankSwitch:Procedure;
                   BytesPerScanLine:Word;
                   {Extended Infos:}
                   XRes:Word;
                   YRes:Word;
                   XCharSize:Byte;
                   YCharSize:Byte;
                   PlaneCount:Byte;
                   BitsPerPixel:Byte;
                   BankCount:Byte;
                   MemModel:Byte;
                   BankSize:Byte;
                   ImagePageCount:Byte;
                   Reserved:Array [0..225] Of Byte;
             End;

Var VideoSeg:Word;
    SwitchAddr:Procedure;
    CurrBank:Word;
    CurrLogScan:Word;
    CurrLogScanBits:Byte;

Implementation

{$L VESA}
Procedure GetVesaInfo;External;
Procedure GetModeInfo;External;
Procedure InitVesaMode;External;
Procedure InitTextMode;External;
Procedure SwitchTo;External;
Procedure SetLogScan;External;
Procedure SetDisplayXY;External;
Procedure PutPixel8;External;
Procedure PutPixel32;External;
Function GetPixel8;External;

Function SetUpMode8;

Begin
     SetUpMode8:=False;
     GetVesaInfo;
     With VesaInfo Do
          Begin
               If (Signature<>'VESA') Then Exit;
               If (Hi(Version)+Lo(Version)/10<1.2) Then Exit;
          End;
     GetModeInfo(ModeNr);
     With ModeInfo Do
          Begin
               If Not(Avail In ModeAttr) Or
                  Not(Graph In ModeAttr) Or
                  Not(BIOSSupport In ModeAttr) Or
                  Not(Col In ModeAttr) Then Exit;
               If Not(Support In WinAAttr) Or
                  Not(Readable In WinAAttr) Or
                  Not(Writeable In WinAAttr) Then Exit;
               If Not(WinGranularity=64) Or
                  Not(WinSize>=64) Then Exit;
               If Not(PlaneCount=1) Or
                  Not(BitsPerPixel=8) Then Exit;
               VideoSeg:=WinASegment;
               SwitchAddr:=BankSwitch;
          End;
     CurrLogScanBits:=1;
     While (1 ShL CurrLogScanBits)<ModeInfo.XRes Do Inc(CurrLogScanBits);
     CurrLogScan:=1 ShL CurrLogScanBits;
     If CurrLogScan*ModeInfo.YRes>LongInt(VesaInfo.VideoRam)*64*1024 Then Exit;
     InitVesaMode(ModeNr);
     SetLogScan(CurrLogScan);
     SetUpMode8:=True;
End;

Function SetUpMode32;

Begin
     SetUpMode32:=False;
     GetVesaInfo;
     With VesaInfo Do
          Begin
               If (Signature<>'VESA') Then Exit;
               If (Hi(Version)+Lo(Version)/10<1.2) Then Exit;
          End;
     GetModeInfo(ModeNr);
     With ModeInfo Do
          Begin
               If Not(Avail In ModeAttr) Or
                  Not(Graph In ModeAttr) Or
                  Not(BIOSSupport In ModeAttr) Or
                  Not(Col In ModeAttr) Then Exit;
               If Not(Support In WinAAttr) Or
                  Not(Readable In WinAAttr) Or
                  Not(Writeable In WinAAttr) Then Exit;
               If Not(WinGranularity=64) Or
                  Not(WinSize>=64) Then Exit;
               If Not(PlaneCount=1) Or
                  Not(BitsPerPixel=32) Then Exit;
               VideoSeg:=WinASegment;
               SwitchAddr:=BankSwitch;
          End;
     CurrLogScanBits:=1;
     While (1 ShL CurrLogScanBits)<ModeInfo.XRes Do Inc(CurrLogScanBits);
     CurrLogScan:=1 ShL CurrLogScanBits;
     If CurrLogScan*ModeInfo.YRes>LongInt(VesaInfo.VideoRam)*64*1024*4 Then Exit;
     InitVesaMode(ModeNr);
     SetLogScan(CurrLogScan);
     SetUpMode32:=True;
End;

Function SetUpMode;

Var I:Integer;

Begin
     SetUpMode:=False;
     GetVesaInfo;
     With VesaInfo Do
          Begin
               If (Signature<>'VESA') Then Exit;
               If (Hi(Version)+Lo(Version)/10<1.2) Then Exit;
          End;
     I:=0;
     While VesaInfo.AvailModes^[I]<>$FFFF Do
           Begin
                GetModeInfo(VesaInfo.AvailModes^[I]);
                With ModeInfo Do
                     Begin
                          If (X=XRes) And
                             (Y=YRes) And
                             (BPB=BitsPerPixel) Then
                             Begin
                                  Case BPB Of
                                       8:SetUpMode:=SetUpMode8(VesaInfo.AvailModes^[I]);
                                       32:SetUpMode:=SetUpMode32(VesaInfo.AvailModes^[I]);
                                  End;
                                  Exit;
                             End;
                     End;
                Inc(I);
           End;
End;

End.